@@ -21,7 +21,7 @@ mod tests {
     #[test]
     fn test_invite_channel() {
         let value = InviteChannel {
-            id: ChannelId(1),
+            id: ChannelId::new(1).expect("non zero"),
             name: Some("channel name".to_owned()),
             kind: ChannelType::GuildText,
         };
